@@ -315,7 +315,7 @@ async def connect_and_consume_rpc(main_config: dict, debug):
     try:
         if debug:
             print_flush("Starting to consume callbacks in connect_and_consume_rpc")
-        task = queue.consume(
+        await queue.consume(
             partial(rabbit_c2_rpc_callback, channel.default_exchange)
         )
     except Exception as e:
@@ -338,7 +338,7 @@ async def mythic_service(debug: bool):
         os.path.abspath(main_config["container_files_path"])
     )
     container_files_path = container_files_path / "c2_code"
-    while True:
+    while connection is None:
         try:
             if debug:
                 print_flush("Connecting to rabbitmq from mythic_service")
@@ -350,45 +350,44 @@ async def mythic_service(debug: bool):
             )
             if debug:
                 print_flush("Successfully connected to rabbitmq from mythic_service")
-            try:
-                channel = await connection.channel()
-                if debug:
-                    print_flush("Declaring exchange in mythic_service")
-                exchange = await channel.declare_exchange(
-                    "mythic_traffic", aio_pika.ExchangeType.TOPIC
-                )
-                if debug:
-                    print_flush("Declaring queue in mythic_service")
-                queue = await channel.declare_queue("", exclusive=True)
-                await queue.bind(
-                    exchange="mythic_traffic", routing_key="c2.modify.{}.#".format(hostname)
-                )
-                # just want to handle one message at a time so we can clean up and be ready
-                await channel.set_qos(prefetch_count=30)
-                if debug:
-                    print_flush("Creating task to consume in mythic_service")
-                task = queue.consume(callback)
-                await sync_classes()
-                print("Listening for c2.modify.{}.#".format(hostname))
-                if debug:
-                    print_flush("Creating task for connect_and_consume_rpc from mythic_service")
-                task4 = asyncio.ensure_future(connect_and_consume_rpc(main_config, debug))
-                if debug:
-                    print_flush("Waiting for all tasks to finish in mythic_service")
-                result = await asyncio.gather(task, task4)
-                if debug:
-                    print_flush("All tasks finished in mythic_service, repeating loop")
-            except Exception as e:
-                print_flush("Exception in mythic_service: " + str(traceback.format_exc()))
         except (ConnectionError, ConnectionRefusedError) as c:
             print_flush("Connection to rabbitmq failed, trying again...")
         except Exception as e:
             print_flush("Exception in mythic_service: " + str(traceback.format_exc()))
         await asyncio.sleep(5)
+    try:
+        channel = await connection.channel()
+        if debug:
+            print_flush("Declaring exchange in mythic_service")
+        exchange = await channel.declare_exchange(
+            "mythic_traffic", aio_pika.ExchangeType.TOPIC
+        )
+        if debug:
+            print_flush("Declaring queue in mythic_service")
+        queue = await channel.declare_queue("", exclusive=True)
+        await queue.bind(
+            exchange="mythic_traffic", routing_key="c2.modify.{}.#".format(hostname)
+        )
+        # just want to handle one message at a time so we can clean up and be ready
+        await channel.set_qos(prefetch_count=30)
+        if debug:
+            print_flush("Creating task to consume in mythic_service")
+        task = queue.consume(callback)
+        await sync_classes()
+        print("Listening for c2.modify.{}.#".format(hostname))
+        if debug:
+            print_flush("Creating task for connect_and_consume_rpc from mythic_service")
+        task4 = asyncio.ensure_future(connect_and_consume_rpc(main_config, debug))
+        if debug:
+            print_flush("Waiting for all tasks to finish in mythic_service")
+        result = await asyncio.gather(task, task4)
+    except Exception as e:
+        print_flush("Exception in mythic_service: " + str(traceback.format_exc()))
     
 
 
 async def heartbeat_loop(debug: bool):
+    connection = None
     config_file = open("rabbitmq_config.json", "rb")
     main_config = json.loads(config_file.read().decode("utf-8"))
     config_file.close()
@@ -396,7 +395,7 @@ async def heartbeat_loop(debug: bool):
         hostname = socket.gethostname()
     else:
         hostname = main_config["name"]
-    while True:
+    while connection is None:
         try:
             if debug:
                 print_flush("Connecting to rabbitmq from heartbeat_loop")
@@ -440,6 +439,5 @@ async def heartbeat_loop(debug: bool):
 # start our service
 def start_service_and_heartbeat(debug=False):
     loop = asyncio.get_event_loop()
-    loop.create_task(mythic_service(debug))
-    loop.create_task(heartbeat_loop(debug))
+    asyncio.gather(mythic_service(debug), heartbeat_loop(debug))
     loop.run_forever()
